@@ -7,7 +7,7 @@
  * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $PostgreSQL: pgsql/src/include/nodes/relation.h,v 1.102 2004/12/31 22:03:34 pgsql Exp $
+ * $PostgreSQL: pgsql/src/include/nodes/relation.h,v 1.119.2.1 2005/11/14 23:54:36 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -24,7 +24,6 @@
  * Relids
  *		Set of relation identifiers (indexes into the rangetable).
  */
-
 typedef Bitmapset *Relids;
 
 /*
@@ -46,6 +45,74 @@ typedef struct QualCost
 	Cost		per_tuple;		/* per-evaluation cost */
 } QualCost;
 
+
+/*----------
+ * PlannerInfo
+ *		Per-query information for planning/optimization
+ *
+ * This struct is conventionally called "root" in all the planner routines.
+ * It holds links to all of the planner's working state, in addition to the
+ * original Query.	Note that at present the planner extensively manipulates
+ * the passed-in Query data structure; someday that should stop.
+ *----------
+ */
+typedef struct PlannerInfo
+{
+	NodeTag		type;
+
+	Query	   *parse;			/* the Query being planned */
+
+	/*
+	 * base_rel_array holds pointers to "base rels" and "other rels" (see
+	 * comments for RelOptInfo for more info).	It is indexed by rangetable
+	 * index (so entry 0 is always wasted).  Entries can be NULL when an RTE
+	 * does not correspond to a base relation.	Note that the array may be
+	 * enlarged on-the-fly.
+	 */
+	struct RelOptInfo **base_rel_array; /* All one-relation RelOptInfos */
+	int			base_rel_array_size;	/* current allocated array len */
+
+	/*
+	 * join_rel_list is a list of all join-relation RelOptInfos we have
+	 * considered in this planning run.  For small problems we just scan the
+	 * list to do lookups, but when there are many join relations we build a
+	 * hash table for faster lookups.  The hash table is present and valid
+	 * when join_rel_hash is not NULL.	Note that we still maintain the list
+	 * even when using the hash table for lookups; this simplifies life for
+	 * GEQO.
+	 */
+	List	   *join_rel_list;	/* list of join-relation RelOptInfos */
+	struct HTAB *join_rel_hash; /* optional hashtable for join relations */
+
+	List	   *equi_key_list;	/* list of lists of equijoined PathKeyItems */
+
+	List	   *left_join_clauses;		/* list of RestrictInfos for outer
+										 * join clauses w/nonnullable var on
+										 * left */
+
+	List	   *right_join_clauses;		/* list of RestrictInfos for outer
+										 * join clauses w/nonnullable var on
+										 * right */
+
+	List	   *full_join_clauses;		/* list of RestrictInfos for full
+										 * outer join clauses */
+
+	List	   *in_info_list;	/* list of InClauseInfos */
+
+	List	   *query_pathkeys; /* desired pathkeys for query_planner(), and
+								 * actual pathkeys afterwards */
+
+	List	   *group_pathkeys; /* groupClause pathkeys, if any */
+	List	   *sort_pathkeys;	/* sortClause pathkeys, if any */
+
+	double		tuple_fraction; /* tuple_fraction passed to query_planner */
+
+	bool		hasJoinRTEs;	/* true if any RTEs are RTE_JOIN kind */
+	bool		hasOuterJoins;	/* true if any RTEs are outer joins */
+	bool		hasHavingQual;	/* true if havingQual was non-null */
+} PlannerInfo;
+
+
 /*----------
  * RelOptInfo
  *		Per-relation information for planning/optimization
@@ -55,16 +122,16 @@ typedef struct QualCost
  * In either case it is uniquely identified by an RT index.  A "joinrel"
  * is the joining of two or more base rels.  A joinrel is identified by
  * the set of RT indexes for its component baserels.  We create RelOptInfo
- * nodes for each baserel and joinrel, and store them in the Query's
- * base_rel_list and join_rel_list respectively.
+ * nodes for each baserel and joinrel, and store them in the PlannerInfo's
+ * base_rel_array and join_rel_list respectively.
  *
  * Note that there is only one joinrel for any given set of component
  * baserels, no matter what order we assemble them in; so an unordered
  * set is the right datatype to identify it with.
  *
  * We also have "other rels", which are like base rels in that they refer to
- * single RT indexes; but they are not part of the join tree, and are stored
- * in other_rel_list not base_rel_list.
+ * single RT indexes; but they are not part of the join tree, and are given
+ * a different RelOptKind to identify them.
  *
  * Currently the only kind of otherrels are those made for child relations
  * of an inheritance scan (SELECT FROM foo*).  The parent table's RTE and
@@ -130,7 +197,7 @@ typedef struct QualCost
  * and joins that the relation participates in:
  *
  *		baserestrictinfo - List of RestrictInfo nodes, containing info about
- *					each qualification clause in which this relation
+ *					each non-join qualification clause in which this relation
  *					participates (only used for base rels)
  *		baserestrictcost - Estimated cost of evaluating the baserestrictinfo
  *					clauses at a single tuple (only used for base rels)
@@ -138,8 +205,8 @@ typedef struct QualCost
  *					side of an outer join, the set of all relids
  *					participating in the highest such outer join; else NULL.
  *					Otherwise, unused.
- *		joininfo  - List of JoinInfo nodes, containing info about each join
- *					clause in which this relation participates
+ *		joininfo  - List of RestrictInfo nodes, containing info about each
+ *					join clause in which this relation participates
  *		index_outer_relids - only used for base rels; set of outer relids
  *					that participate in indexable joinclauses for this rel
  *		index_inner_paths - only used for base rels; list of InnerIndexscanInfo
@@ -155,7 +222,7 @@ typedef struct QualCost
  * and should not be processed again at the level of {1 2 3}.)	Therefore,
  * the restrictinfo list in the join case appears in individual JoinPaths
  * (field joinrestrictinfo), not in the parent relation.  But it's OK for
- * the RelOptInfo to store the joininfo lists, because those are the same
+ * the RelOptInfo to store the joininfo list, because that is the same
  * for a given rel no matter how we form it.
  *
  * We store baserestrictcost in the RelOptInfo (for base relations) because
@@ -202,16 +269,17 @@ typedef struct RelOptInfo
 	Relids	   *attr_needed;	/* array indexed [min_attr .. max_attr] */
 	int32	   *attr_widths;	/* array indexed [min_attr .. max_attr] */
 	List	   *indexlist;
-	BlockNumber	pages;
+	BlockNumber pages;
 	double		tuples;
 	struct Plan *subplan;		/* if subquery */
 
 	/* used by various scans and joins: */
-	List	   *baserestrictinfo;		/* RestrictInfo structures (if
-										 * base rel) */
+	List	   *baserestrictinfo;		/* RestrictInfo structures (if base
+										 * rel) */
 	QualCost	baserestrictcost;		/* cost of evaluating the above */
 	Relids		outerjoinset;	/* set of base relids */
-	List	   *joininfo;		/* JoinInfo structures */
+	List	   *joininfo;		/* RestrictInfo structures for join clauses
+								 * involving this rel */
 
 	/* cached info about inner indexscan paths for relation: */
 	Relids		index_outer_relids;		/* other relids in indexable join
@@ -220,10 +288,9 @@ typedef struct RelOptInfo
 
 	/*
 	 * Inner indexscans are not in the main pathlist because they are not
-	 * usable except in specific join contexts.  We use the
-	 * index_inner_paths list just to avoid recomputing the best inner
-	 * indexscan repeatedly for similar outer relations.  See comments for
-	 * InnerIndexscanInfo.
+	 * usable except in specific join contexts.  We use the index_inner_paths
+	 * list just to avoid recomputing the best inner indexscan repeatedly for
+	 * similar outer relations.  See comments for InnerIndexscanInfo.
 	 */
 } RelOptInfo;
 
@@ -253,9 +320,10 @@ typedef struct IndexOptInfo
 	NodeTag		type;
 
 	Oid			indexoid;		/* OID of the index relation */
+	RelOptInfo *rel;			/* back-link to index's table */
 
 	/* statistics from pg_class */
-	BlockNumber	pages;			/* number of disk pages in index */
+	BlockNumber pages;			/* number of disk pages in index */
 	double		tuples;			/* number of index tuples in index */
 
 	/* index descriptor information */
@@ -267,16 +335,12 @@ typedef struct IndexOptInfo
 
 	RegProcedure amcostestimate;	/* OID of the access method's cost fcn */
 
-	List	   *indexprs;		/* expressions for non-simple index
-								 * columns */
+	List	   *indexprs;		/* expressions for non-simple index columns */
 	List	   *indpred;		/* predicate if a partial index, else NIL */
 
 	bool		predOK;			/* true if predicate matches query */
 	bool		unique;			/* true if a unique index */
-
-	/* cached info about inner indexscan paths for index */
-	Relids		outer_relids;	/* other relids in usable join clauses */
-	List	   *inner_paths;	/* List of InnerIndexscanInfo nodes */
+	bool		amoptionalkey;	/* can query omit key for the first column? */
 } IndexOptInfo;
 
 
@@ -288,8 +352,8 @@ typedef struct IndexOptInfo
  *	the first sublist represents the primary sort key, the second the
  *	first secondary sort key, etc.	Each sublist contains one or more
  *	PathKeyItem nodes, each of which can be taken as the attribute that
- *	appears at that sort position.	(See the top of optimizer/path/pathkeys.c
- *	for more information.)
+ *	appears at that sort position.	(See optimizer/README for more
+ *	information.)
  */
 
 typedef struct PathKeyItem
@@ -300,9 +364,9 @@ typedef struct PathKeyItem
 	Oid			sortop;			/* the ordering operator ('<' op) */
 
 	/*
-	 * key typically points to a Var node, ie a relation attribute, but it
-	 * can also point to an arbitrary expression representing the value
-	 * indexed by an index expression.
+	 * key typically points to a Var node, ie a relation attribute, but it can
+	 * also point to an arbitrary expression representing the value indexed by
+	 * an index expression.
 	 */
 } PathKeyItem;
 
@@ -325,40 +389,28 @@ typedef struct Path
 	RelOptInfo *parent;			/* the relation this path can build */
 
 	/* estimated execution costs for path (see costsize.c for more info) */
-	Cost		startup_cost;	/* cost expended before fetching any
-								 * tuples */
-	Cost		total_cost;		/* total cost (assuming all tuples
-								 * fetched) */
+	Cost		startup_cost;	/* cost expended before fetching any tuples */
+	Cost		total_cost;		/* total cost (assuming all tuples fetched) */
 
 	List	   *pathkeys;		/* sort ordering of path's output */
 	/* pathkeys is a List of Lists of PathKeyItem nodes; see above */
 } Path;
 
 /*----------
- * IndexPath represents an index scan.	Although an indexscan can only read
- * a single relation, it can scan it more than once, potentially using a
- * different index during each scan.  The result is the union (OR) of all the
- * tuples matched during any scan.	(The executor is smart enough not to return
- * the same tuple more than once, even if it is matched in multiple scans.)
+ * IndexPath represents an index scan over a single index.
  *
- * 'indexinfo' is a list of IndexOptInfo nodes, one per scan to be performed.
+ * 'indexinfo' is the index to be scanned.
  *
- * 'indexclauses' is a list of index qualifications, also one per scan.
- * Each entry in 'indexclauses' is a sublist of qualification clauses to be
- * used for that scan, with implicit AND semantics across the sublist items.
- * NOTE that the semantics of the top-level list in 'indexclauses' is OR
- * combination, while the sublists are implicitly AND combinations!
+ * 'indexclauses' is a list of index qualification clauses, with implicit
+ * AND semantics across the list.  Each clause is a RestrictInfo node from
+ * the query's WHERE or JOIN conditions.
  *
  * 'indexquals' has the same structure as 'indexclauses', but it contains
- * the actual indexqual conditions that can be used with the index(es).
+ * the actual indexqual conditions that can be used with the index.
  * In simple cases this is identical to 'indexclauses', but when special
  * indexable operators appear in 'indexclauses', they are replaced by the
  * derived indexscannable conditions in 'indexquals'.
  *
- * Both 'indexclauses' and 'indexquals' are lists of sublists of RestrictInfo
- * nodes.  (Before 8.0, we kept bare operator expressions in these lists, but
- * storing RestrictInfos is more efficient since selectivities can be cached.)
- *
  * 'isjoininner' is TRUE if the path is a nestloop inner scan (that is,
  * some of the index conditions are join rather than restriction clauses).
  *
@@ -370,6 +422,11 @@ typedef struct Path
  * NoMovementScanDirection for an indexscan, but the planner wants to
  * distinguish ordered from unordered indexes for building pathkeys.)
  *
+ * 'indextotalcost' and 'indexselectivity' are saved in the IndexPath so that
+ * we need not recompute them when considering using the same index in a
+ * bitmap index/heap scan (see BitmapHeapPath).  The costs of the IndexPath
+ * itself represent the costs of an IndexScan plan type.
+ *
  * 'rows' is the estimated result tuple count for the indexscan.  This
  * is the same as path.parent->rows for a simple indexscan, but it is
  * different for a nestloop inner scan, because the additional indexquals
@@ -380,14 +437,69 @@ typedef struct Path
 typedef struct IndexPath
 {
 	Path		path;
-	List	   *indexinfo;
+	IndexOptInfo *indexinfo;
 	List	   *indexclauses;
 	List	   *indexquals;
 	bool		isjoininner;
 	ScanDirection indexscandir;
+	Cost		indextotalcost;
+	Selectivity indexselectivity;
 	double		rows;			/* estimated number of result tuples */
 } IndexPath;
 
+/*
+ * BitmapHeapPath represents one or more indexscans that generate TID bitmaps
+ * instead of directly accessing the heap, followed by AND/OR combinations
+ * to produce a single bitmap, followed by a heap scan that uses the bitmap.
+ * Note that the output is always considered unordered, since it will come
+ * out in physical heap order no matter what the underlying indexes did.
+ *
+ * The individual indexscans are represented by IndexPath nodes, and any
+ * logic on top of them is represented by a tree of BitmapAndPath and
+ * BitmapOrPath nodes.	Notice that we can use the same IndexPath node both
+ * to represent a regular IndexScan plan, and as the child of a BitmapHeapPath
+ * that represents scanning the same index using a BitmapIndexScan.  The
+ * startup_cost and total_cost figures of an IndexPath always represent the
+ * costs to use it as a regular IndexScan.	The costs of a BitmapIndexScan
+ * can be computed using the IndexPath's indextotalcost and indexselectivity.
+ *
+ * BitmapHeapPaths can be nestloop inner indexscans.  The isjoininner and
+ * rows fields serve the same purpose as for plain IndexPaths.
+ */
+typedef struct BitmapHeapPath
+{
+	Path		path;
+	Path	   *bitmapqual;		/* IndexPath, BitmapAndPath, BitmapOrPath */
+	bool		isjoininner;	/* T if it's a nestloop inner scan */
+	double		rows;			/* estimated number of result tuples */
+} BitmapHeapPath;
+
+/*
+ * BitmapAndPath represents a BitmapAnd plan node; it can only appear as
+ * part of the substructure of a BitmapHeapPath.  The Path structure is
+ * a bit more heavyweight than we really need for this, but for simplicity
+ * we make it a derivative of Path anyway.
+ */
+typedef struct BitmapAndPath
+{
+	Path		path;
+	List	   *bitmapquals;	/* IndexPaths and BitmapOrPaths */
+	Selectivity bitmapselectivity;
+} BitmapAndPath;
+
+/*
+ * BitmapOrPath represents a BitmapOr plan node; it can only appear as
+ * part of the substructure of a BitmapHeapPath.  The Path structure is
+ * a bit more heavyweight than we really need for this, but for simplicity
+ * we make it a derivative of Path anyway.
+ */
+typedef struct BitmapOrPath
+{
+	Path		path;
+	List	   *bitmapquals;	/* IndexPaths and BitmapAndPaths */
+	Selectivity bitmapselectivity;
+} BitmapOrPath;
+
 /*
  * TidPath represents a scan by TID
  *
@@ -404,6 +516,9 @@ typedef struct TidPath
  * AppendPath represents an Append plan, ie, successive execution of
  * several member plans.  Currently it is only used to handle expansion
  * of inheritance trees.
+ *
+ * Note: it is possible for "subpaths" to contain only one, or even no,
+ * elements.  These cases are optimized during create_append_plan.
  */
 typedef struct AppendPath
 {
@@ -412,10 +527,17 @@ typedef struct AppendPath
 } AppendPath;
 
 /*
- * ResultPath represents use of a Result plan node, either to compute a
- * variable-free targetlist or to gate execution of a subplan with a
- * one-time (variable-free) qual condition.  Note that in the former case
- * path.parent will be NULL; in the latter case it is copied from the subpath.
+ * ResultPath represents use of a Result plan node.  There are several
+ * applications for this:
+ *	* To compute a variable-free targetlist (a "SELECT expressions" query).
+ *	  In this case subpath and path.parent will both be NULL.  constantqual
+ *	  might or might not be empty ("SELECT expressions WHERE something").
+ *	* To gate execution of a subplan with a one-time (variable-free) qual
+ *	  condition.  path.parent is copied from the subpath.
+ *	* To substitute for a scan plan when we have proven that no rows in
+ *	  a table will satisfy the query.  subpath is NULL but path.parent
+ *	  references the not-to-be-scanned relation, and constantqual is
+ *	  a constant FALSE.
  *
  * Note that constantqual is a list of bare clauses, not RestrictInfos.
  */
@@ -513,8 +635,7 @@ typedef JoinPath NestPath;
 typedef struct MergePath
 {
 	JoinPath	jpath;
-	List	   *path_mergeclauses;		/* join clauses to be used for
-										 * merge */
+	List	   *path_mergeclauses;		/* join clauses to be used for merge */
 	List	   *outersortkeys;	/* keys for explicit sort, if any */
 	List	   *innersortkeys;	/* keys for explicit sort, if any */
 } MergePath;
@@ -547,8 +668,8 @@ typedef struct HashPath
  * in the baserestrictinfo list of the RelOptInfo for that base rel.
  *
  * If a restriction clause references more than one base rel, it will
- * appear in the JoinInfo lists of every RelOptInfo that describes a strict
- * subset of the base rels mentioned in the clause.  The JoinInfo lists are
+ * appear in the joininfo list of every RelOptInfo that describes a strict
+ * subset of the base rels mentioned in the clause.  The joininfo lists are
  * used to drive join tree building by selecting plausible join candidates.
  * The clause cannot actually be applied until we have built a join rel
  * containing all the base rels it references, however.
@@ -578,8 +699,8 @@ typedef struct HashPath
  * pushed down to a lower level than its original syntactic placement in the
  * join tree would suggest.  If an outer join prevents us from pushing a qual
  * down to its "natural" semantic level (the level associated with just the
- * base rels used in the qual) then the qual will appear in JoinInfo lists
- * that reference more than just the base rels it actually uses.  By
+ * base rels used in the qual) then we mark the qual with a "required_relids"
+ * value including more than just the base rels it actually uses.  By
  * pretending that the qual references all the rels appearing in the outer
  * join, we prevent it from being evaluated below the outer join's joinrel.
  * When we do form the outer join's joinrel, we still need to distinguish
@@ -587,15 +708,16 @@ typedef struct HashPath
  * that appeared higher in the tree and were pushed down to the join rel
  * because they used no other rels.  That's what the is_pushed_down flag is
  * for; it tells us that a qual came from a point above the join of the
- * specific set of base rels that it uses (or that the JoinInfo structures
- * claim it uses).	A clause that originally came from WHERE will *always*
- * have its is_pushed_down flag set; a clause that came from an INNER JOIN
- * condition, but doesn't use all the rels being joined, will also have
- * is_pushed_down set because it will get attached to some lower joinrel.
+ * set of base rels listed in required_relids.	A clause that originally came
+ * from WHERE will *always* have its is_pushed_down flag set; a clause that
+ * came from an INNER JOIN condition, but doesn't use all the rels being
+ * joined, will also have is_pushed_down set because it will get attached to
+ * some lower joinrel.
  *
- * We also store a valid_everywhere flag, which says that the clause is not
- * affected by any lower-level outer join, and therefore any conditions it
- * asserts can be presumed true throughout the plan tree.
+ * When application of a qual must be delayed by outer join, we also mark it
+ * with outerjoin_delayed = true.  This isn't redundant with required_relids
+ * because that might equal clause_relids whether or not it's an outer-join
+ * clause.
  *
  * In general, the referenced clause might be arbitrarily complex.	The
  * kinds of clauses we can handle as indexscan quals, mergejoin clauses,
@@ -623,20 +745,23 @@ typedef struct RestrictInfo
 
 	bool		is_pushed_down; /* TRUE if clause was pushed down in level */
 
-	bool		valid_everywhere;		/* TRUE if valid on every level */
+	bool		outerjoin_delayed;		/* TRUE if delayed by outer join */
 
 	/*
-	 * This flag is set true if the clause looks potentially useful as a
-	 * merge or hash join clause, that is if it is a binary opclause with
-	 * nonoverlapping sets of relids referenced in the left and right
-	 * sides. (Whether the operator is actually merge or hash joinable
-	 * isn't checked, however.)
+	 * This flag is set true if the clause looks potentially useful as a merge
+	 * or hash join clause, that is if it is a binary opclause with
+	 * nonoverlapping sets of relids referenced in the left and right sides.
+	 * (Whether the operator is actually merge or hash joinable isn't checked,
+	 * however.)
 	 */
 	bool		can_join;
 
-	/* The set of relids (varnos) referenced in the clause: */
+	/* The set of relids (varnos) actually referenced in the clause: */
 	Relids		clause_relids;
 
+	/* The set of relids required to evaluate the clause: */
+	Relids		required_relids;
+
 	/* These fields are set for any binary opclause: */
 	Relids		left_relids;	/* relids in left side of clause */
 	Relids		right_relids;	/* relids in right side of clause */
@@ -669,27 +794,6 @@ typedef struct RestrictInfo
 	Selectivity right_bucketsize;		/* avg bucketsize of right side */
 } RestrictInfo;
 
-/*
- * Join clause info.
- *
- * We make a list of these for each RelOptInfo, containing info about
- * all the join clauses this RelOptInfo participates in.  (For this
- * purpose, a "join clause" is a WHERE clause that mentions both vars
- * belonging to this relation and vars belonging to relations not yet
- * joined to it.)  We group these clauses according to the set of
- * other base relations (unjoined relations) mentioned in them.
- * There is one JoinInfo for each distinct set of unjoined_relids,
- * and its jinfo_restrictinfo lists the clause(s) that use that set
- * of other relations.
- */
-
-typedef struct JoinInfo
-{
-	NodeTag		type;
-	Relids		unjoined_relids;	/* some rels not yet part of my RelOptInfo */
-	List	   *jinfo_restrictinfo;		/* relevant RestrictInfos */
-} JoinInfo;
-
 /*
  * Inner indexscan info.
  *
@@ -701,16 +805,13 @@ typedef struct JoinInfo
  * thus varies depending on which outer relation we consider; so we have
  * to recompute the best such path for every join.	To avoid lots of
  * redundant computation, we cache the results of such searches.  For
- * each index we compute the set of possible otherrelids (all relids
- * appearing in joinquals that could become indexquals for this index).
+ * each relation we compute the set of possible otherrelids (all relids
+ * appearing in joinquals that could become indexquals for this table).
  * Two outer relations whose relids have the same intersection with this
  * set will have the same set of available joinclauses and thus the same
- * best inner indexscan for that index.  Similarly, for each base relation,
- * we form the union of the per-index otherrelids sets.  Two outer relations
- * with the same intersection with that set will have the same best overall
- * inner indexscan for the base relation.  We use lists of InnerIndexscanInfo
- * nodes to cache the results of these searches at both the index and
- * relation level.
+ * best inner indexscan for the inner relation.  By taking the intersection
+ * before scanning the cache, we avoid recomputing when considering
+ * join rels that differ only by the inclusion of irrelevant other rels.
  *
  * The search key also includes a bool showing whether the join being
  * considered is an outer join.  Since we constrain the join order for
@@ -734,7 +835,7 @@ typedef struct InnerIndexscanInfo
  * When we convert top-level IN quals into join operations, we must restrict
  * the order of joining and use special join methods at some join points.
  * We record information about each such IN clause in an InClauseInfo struct.
- * These structs are kept in the Query node's in_info_list.
+ * These structs are kept in the PlannerInfo node's in_info_list.
  */
 
 typedef struct InClauseInfo
@@ -745,8 +846,8 @@ typedef struct InClauseInfo
 	List	   *sub_targetlist; /* targetlist of original RHS subquery */
 
 	/*
-	 * Note: sub_targetlist is just a list of Vars or expressions; it does
-	 * not contain TargetEntry nodes.
+	 * Note: sub_targetlist is just a list of Vars or expressions; it does not
+	 * contain TargetEntry nodes.
 	 */
 } InClauseInfo;
 
